import 'dart:async';
import 'package:flutter/material.dart';

extension FutureX<T> on Future<T> {
  Stream<T> toStream() => Stream.fromFuture(this);

  Widget toWidget({
    required Widget Function(T data) onData,
    Widget Function()? onEmpty,
    T? initialData,
  }) =>
      FutureBuilder<T>(
        future: this,
        initialData: initialData,
        builder: (_, s) =>
            s.hasData ? onData(s.data!) : onEmpty?.call() ?? SizedBox(),
      );
}

Future postFrameFuture(FutureOr Function() callback) {
  final c = Completer();
  WidgetsBinding.instance?.addPostFrameCallback((_) async {
    await callback();
    c.complete();
  });
  return c.future;
}
