import 'dart:async';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';
import 'package:rxdart/rxdart.dart' as Reactive;

extension GetExtension<T> on Rx<T> {
  ObxValue<Rx<T>> toWidget(
    Widget Function(Rx<T> rx) builder,
  ) =>
      ObxValue<Rx<T>>(builder, this);

  StreamBuilder<E?> toMappedWidget<E>(
    E? Function(T model) mapper,
    Widget Function(E? item) builder,
  ) =>
      StreamBuilder<E?>(
        initialData: null,
        stream: initialValueStreamMap(mapper),
        builder: (_, s) => builder(s.data),
      );
}

extension GetExtensionX<Request, Response>
    on RxQueryPageable<Request, Response> {
  ObxValue<RxQueryPageable<Request, Response>> toWidget(
    Widget Function(RxQueryPageable<Request, Response> rx) builder,
  ) =>
      ObxValue<RxQueryPageable<Request, Response>>(builder, this);
}

extension RxBoolX on RxBool {
  ObxValue<RxBool> toOpacityWidget(
    Widget Function(RxBool rx) builder,
  ) =>
      ObxValue<RxBool>(
        (rx) => Opacity(
          opacity: rx.value ? 0.3 : 1,
          child: builder(rx),
        ),
        this,
      );
}

extension GetReactiveExtension<T> on Rx<T> {
  StreamSubscription<T> listenWithInitialValue(
    void Function(T data) onData,
  ) =>
      initialValueStream().listen(onData);

  Stream<T> initialValueStream() => subject.stream
      .asBroadcastStream()
      .mergeWith([Future.value(value).toStream()]).distinct();

  Stream<E> initialValueStreamMap<E>(
    E Function(T item) mapper,
  ) =>
      initialValueStream().map(mapper);
}

extension RxPageableX<Request, Response> on RxQueryPageable<Request, Response> {
  RxPageableList<Request, Response> toPageableList({
    required Widget Function(Response item) builder,
    bool reverse = false,
    bool shrinkWrap = false,
    Widget? header,
    Widget? separator,
    Widget? empty,
    EdgeInsetsGeometry? padding,
    Axis scrollDirection = Axis.vertical,
    VoidCallback? next,
  }) =>
      RxPageableList<Request, Response>(
        builder: builder,
        source: this,
        reverse: reverse,
        shrinkWrap: shrinkWrap,
        header: header,
        separator: separator,
        empty: empty,
        padding: padding,
        scrollDirection: scrollDirection,
        next: next,
      );
}
