import 'dart:async';

import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

class PageableRequest {
  int page;
  int size;

  PageableRequest({
    required this.page,
    required this.size,
  });
}

class RxQueryPageable<Query, Response> extends RxList<Response>
    implements RxAutoStartable {
  static var pageSize = 20;
  static var defaultFirstPage = 1;

  Future<List<Response>> Function(Query query, PageableRequest request)
      dataSource;
  Query query;
  bool isAutoStart;

  RxQueryPageable({
    required this.query,
    required this.dataSource,
    this.isAutoStart = true,
  }) : super() {
    if (isAutoStart) {
      reloadDataSource();
    }
  }

  late int _currentPage = defaultFirstPage;
  late bool _isEnded = false;
  late bool _isLoading = false;

  String get currentStatus =>
      "isLoading: $_isLoading, isEnded: $_isEnded, currentPage: $_currentPage";

  Future reloadDataSource({
    Query? updatedQuery,
  }) async {
    if (updatedQuery != null) {
      query = updatedQuery;
    }

    isAutoStart = true;

    clear();
    _currentPage = defaultFirstPage;
    _isEnded = false;
    _isLoading = false;

    await next().timeout(Duration(seconds: 10));
  }

  Response item(int index) {
    final half = pageSize / 2;
    if (index + half > value.length) {
      next();
    }
    return value[index];
  }

  Future next() async {
    if (_isEnded || _isLoading) return;
    _isLoading = true;
    return postFrameFuture(() async {
      try {
        final list = await dataSource(
          query,
          PageableRequest(page: _currentPage, size: pageSize),
        );
        if (_currentPage == defaultFirstPage) {
          clear();
        }
        addAllIf(true, list.toList());
        _currentPage += 1;
        _isEnded = pageSize > list.length;
      } finally {
        _isLoading = false;
      }
    });
  }
}

class RxPageable<Response> extends RxQueryPageable<void, Response> {
  RxPageable({
    required Future<List<Response>> Function(PageableRequest request)
        dataSource,
    bool isAutoStart = true,
  }) : super(
          query: {},
          dataSource: (_, request) => dataSource(request),
          isAutoStart: isAutoStart,
        );
}
